<?php

namespace Storage;

use Psr\Http\Message\RequestInterface,
    Psr\Http\Message\ResponseInterface,
    Psr\Http\Server\MiddlewareInterface,
    Psr\Http\Message\ServerRequestInterface,
    Psr\Http\Server\RequestHandlerInterface;

class SessionMiddleware implements MiddlewareInterface
{
    protected array $options = [
        'name' => '',
        'path' => null,
        'domain' => null,
        'secure' => false,
        'httponly' => true,
        'lifetime' => 43200,  # A half-day
        'samesite' => "",
    ];

    protected Cookie $cookie;

    protected string $savePath;

    public function __construct(Cookie $cookie, array $options = [], ?string $savePath = null)
    {
        $this->cookie = $cookie;

        if ( empty($options['name']) ) {
            # unique directory-based named session
            $options['name'] = md5(__FILE__);
        }

        $this->options = array_merge($this->options, array_filter(array_intersect_key($this->cookie->options, array_flip(['path', 'domain', 'samesite']))), $options);

        $this->savePath = $savePath ?? sys_get_temp_dir();
    }

    /**
     *
     * @param  RequestInterface        $request  PSR7 request object
     * @param  RequestHandlerInterface $response PSR7 handler object
     *
     * @return ResponseInterface PSR7 response object
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler) : ResponseInterface
    {
        Session::start($this->options, $this->cookie, $this->savePath);

        return $handler->handle($request);
    }
}
